=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2010-01-12 06:12:31 +0000
+++ bzrlib/repository.py	2010-01-14 06:43:27 +0000
@@ -4591,11 +4591,26 @@
 
     def _get_convertable_inventory_stream(self, revision_ids,
                                           delta_versus_null=False):
-        # The source is using CHKs, but the target either doesn't or it has a
-        # different serializer.  The StreamSink code expects to be able to
+        # The two formats are sufficiently different that there is no fast
+        # path, so we need to send just inventorydeltas, which any
+        # sufficiently modern client can insert into any repository.
+        # The StreamSink code expects to be able to
         # convert on the target, so we need to put bytes-on-the-wire that can
         # be converted.  That means inventory deltas (if the remote is <1.19,
         # RemoteStreamSink will fallback to VFS to insert the deltas).
+        # 
+        # See <https://launchpad.net/bugs/456077> asking for a warning here
+        #
+        # Note that for the smart server this runs on the server, so for ssh
+        # it will be sent back over stderr, and for tcp and http it will
+        # probably be lost.
+        trace.warning("Using format-conversion stream\n"
+            "from %s to %s.\n"
+            "This may take some time. Upgrade the branches to the same format \n"
+            "for better results.\n"
+            % (self.from_repository._format,
+               self.to_format))
+
         yield ('inventory-deltas',
            self._stream_invs_as_deltas(revision_ids,
                                        delta_versus_null=delta_versus_null))

=== modified file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	2010-01-12 06:12:31 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2010-01-14 06:43:27 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,13 +20,18 @@
 import os
 import sys
 
+from bzrlib import (
+    debug,
+    remote,
+    urlutils,
+    )
+
 from bzrlib.branch import Branch
 from bzrlib.directory_service import directories
 from bzrlib.osutils import pathjoin
 from bzrlib.tests.blackbox import ExternalBase
 from bzrlib.uncommit import uncommit
 from bzrlib.workingtree import WorkingTree
-from bzrlib import urlutils
 
 
 class TestPull(ExternalBase):
@@ -394,10 +399,37 @@
     def test_pull_cross_format_warning(self):
         """You get a warning for probably slow cross-format pulls.
         """
-
+        # this is assumed to be going through InterDifferingSerializer
         from_tree = self.make_branch_and_tree('from', format='2a')
         to_tree = self.make_branch_and_tree('to', format='1.14-rich-root')
         from_tree.commit(message='first commit')
         out, err = self.run_bzr(['pull', '-d', 'to', 'from'])
         self.assertContainsRe(err,
             "(?m)Fetching between repositories with different formats.*")
+
+    def test_pull_cross_format_warning_no_IDS(self):
+        """You get a warning for probably slow cross-format pulls.
+        """
+        # this simulates what would happen across the network, where
+        # interdifferingserializer is not active
+
+        debug.debug_flags.add('IDS_never')
+        # TestCase take care of restoring them
+
+        from_tree = self.make_branch_and_tree('from', format='2a')
+        to_tree = self.make_branch_and_tree('to', format='1.14-rich-root')
+        from_tree.commit(message='first commit')
+        out, err = self.run_bzr(['pull', '-d', 'to', 'from'])
+        self.assertContainsRe(err,
+            "(?m)Using format-conversion stream*")
+
+    def test_pull_cross_format_from_network(self):
+        self.setup_smart_server_with_call_log()
+        from_tree = self.make_branch_and_tree('from', format='2a')
+        to_tree = self.make_branch_and_tree('to', format='1.14-rich-root')
+        self.assertIsInstance(from_tree.branch, remote.RemoteBranch)
+        from_tree.commit(message='first commit')
+        out, err = self.run_bzr(['pull', '-d', 'to',
+            from_tree.branch.bzrdir.root_transport.base])
+        self.assertContainsRe(err,
+            "(?m)Using format-conversion stream*")

